import { useState, useRef, useEffect, ReactNode } from 'react'
import {
  DtToast,
  DtPage,
  DtLoading,
  alertLayer,
  toastLayer,
} from '@dt/ui-mobile'
import NovuWarpper from './NovuWarpper'
import { alertCircle, chevronRight } from '@dt/ui-ionic-icon'
import { observer } from 'mobx-react'
import { Clipboard } from '@dt/fe-capacitor'
import { pageSpy } from '@dt/fe-base'
import { useLocation } from 'react-router-dom'
import { useMqttInit } from '@init/mqtt'
import MenuView from './MenuView'
import NetworkError from './NetworkError'
import NoLoginPermissionModal from './NoLoginPermissionModal'
import Outlet from './Outlet'
import { appMobx } from '@mobx'
import i18n from '@dt/fe-base/dist/i18n'
import { ErrorBoundary } from '@dt/ui-mobile-business'
import './index.scss'
import { setHistoryNew, useHistoryNew } from '@common/hooks'

const Dashboard: React.FC<{ children?: ReactNode }> = () => {
  const {
    showDebug,
    setShowDebug,
    showOnlineDebug,
    setShowOnlineDebug,
    setCurPath,
    setPrePath,
    pageLoading,
  } = appMobx
  const debugCountRef = useRef<number>(0)
  const onlineDebugCountRef = useRef<number>(0)
  const [showNetWorkError, setShowNetWorkError] = useState(false)
  const location = useLocation()

  const history = useHistoryNew()
  setHistoryNew(history)

  // 打开调试
  const handleDebugClick = () => {
    const count = debugCountRef.current + 1
    if (count === 10) {
      if (!showDebug) {
        toastLayer.success('调试模式打开中...')
      }
      setShowDebug(!showDebug)
      debugCountRef.current = 0
    } else {
      debugCountRef.current = count
    }
  }

  // 打开在线调试
  const handleOnlineDebugClick = () => {
    const count = onlineDebugCountRef.current + 1
    if (count === 10) {
      if (!showOnlineDebug) {
        const address = pageSpy.getAddress()
        Clipboard.write({ string: address })
        alertLayer({
          backdropDismiss: false,
          header: '提示',
          message: `调试码 ${address} 已复制到剪切板，发送给客服开启在线调试`,
          button: i18n.t('common.ikonw'),
        })
      } else {
        toastLayer.success('在线调试模式已关闭')
      }
      setShowOnlineDebug(!showOnlineDebug)
      onlineDebugCountRef.current = 0
    } else {
      onlineDebugCountRef.current = count
    }
  }

  // app 升级
  const handleUpgrade = () => {
    // AppMobx.setShowUpgradeModal(false)
    // Browser.open({ url: AppMobx.upgradeInfo.downloadUrl })
  }

  // 监听路由变化
  useEffect(() => {
    debugCountRef.current = 0
    onlineDebugCountRef.current = 0
    if (showOnlineDebug) {
      pageSpy.isExpired() && setShowOnlineDebug(false)
    }
    setCurPath(location.pathname)

    return () => {
      setPrePath(location.pathname)
    }
  }, [location])

  useMqttInit()

  return (
    <NovuWarpper>
      {/* 左侧导航栏 */}
      <MenuView>
        <div className={'btn-debug'} onClick={handleDebugClick}></div>
        <div
          className={'btn-online-debug'}
          onClick={handleOnlineDebugClick}
        ></div>
      </MenuView>
      {/* 主页面 */}
      <DtPage id='mobile-hq-page-content'>
        {/* 路由渲染 */}
        {/* <KeepAliveProvider>
          <AnimatedRouter />
        </KeepAliveProvider> */}
        <ErrorBoundary>
          <Outlet />
        </ErrorBoundary>
        {pageLoading && <DtLoading duration={10000} isOpen={pageLoading} />}
      </DtPage>
      {/* 网络异常 toast */}
      <DtToast
        isOpen={!showNetWorkError && appMobx.networkError}
        color='danger'
        position='top'
        icon={alertCircle}
        message={i18n.t('system.netErrorCheck')}
        buttons={[
          {
            icon: chevronRight,
            handler: () => setShowNetWorkError(true),
          },
        ]}
      />
      {/* 网络异常 弹窗 */}
      {showNetWorkError && appMobx.networkError && (
        <NetworkError isOpen onBack={() => setShowNetWorkError(false)} />
      )}
      <NoLoginPermissionModal />
    </NovuWarpper>
  )
}

export default observer(Dashboard)
