import React, { FC, Fragment, useEffect, useState } from 'react'
import { DtButton } from '@dt/ui-mobile'
import Icon from '../../elements/Icon'
import TimeSelectModal from './modals/timeSelectModal'
import { classNames } from '@dt/fe-base'
import i18n from '@dt/fe-base/dist/i18n'
import { IDate, validateTime } from './utils'
import './index.scss'

interface IBusinessTimeProps {
  /** 抬头 */
  title: string
  /** 编辑时候的描述 */
  editDesc?: string
  dataList: IDate[]
  timeRange?: IDate
  onConfirm: (value: IDate[]) => void
}

const BusinessTime: FC<IBusinessTimeProps> = (props) => {
  const _ns = 'time-line'
  const {
    title,
    editDesc,
    dataList,
    onConfirm,
    timeRange = { startTime: '00:00', endTime: '23:59' },
  } = props
  const [isEdit, setIsEdit] = useState<boolean>(false)
  const [editIndex, setEditIndex] = useState<number | undefined>()
  const [isStart, setIsStart] = useState<boolean>(false)
  const [editVal, setEditVal] = useState<string | undefined>('00:00')
  const [timeSelectVisible, setTimeSelectVisible] = useState<boolean>(false)
  const [list, setList] = useState<IDate[]>([])
  const [canChoose, setCanChoose] = useState<{
    start: string
    middle?: boolean
    end: string
  }>({ start: '00:00', end: '23:59' })

  useEffect(() => {
    setList([...dataList])
  }, [dataList])

  useEffect(() => {
    setCanChoose({
      start: timeRange.startTime,
      middle: timeRange.startTime > timeRange.endTime,
      end: timeRange.endTime,
    })
  }, [timeRange, dataList])

  const timeChange = (value: string) => {
    const arr = [...list]
    if (isStart) {
      arr[editIndex || 0] = { ...arr[editIndex || 0], startTime: value }
    } else {
      arr[editIndex || 0] = { ...arr[editIndex || 0], endTime: value }
    }
    if (validateTime(arr, timeRange)) {
      setList([...arr])
    }
  }

  const editClick = (index: number, start: boolean, val: string) => {
    setIsStart(start)
    setEditIndex(index)
    setTimeSelectVisible(true)
    setEditVal(val)
  }

  const addRange = () => {
    if (list[list.length - 1].endTime < canChoose.end) {
      setList([
        ...list,
        { startTime: list[list.length - 1].endTime, endTime: canChoose.end },
      ])
    }
  }

  const delRange = (index: number) => {
    if (list.length > 1) {
      const arr = [...list]
      arr.splice(index, 1)
      setList([...arr])
    }
  }

  return (
    <>
      <div
        className={_ns}
        onClick={() => {
          if (!isEdit) {
            setIsEdit(true)
          }
        }}
      >
        <div className={`${_ns}-title`}>
          {title}
          {isEdit && <div className='desc'>{editDesc}</div>}
        </div>
        {isEdit ? (
          <div>
            <div style={{ maxHeight: '330px', overflowY: 'auto' }}>
              {list.map((item, index) => (
                <div key={index}>
                  <div className='del-line'>
                    <div className='text'>
                      {i18n.t('businessHour.timePeriod', { num: index + 1 })}
                    </div>
                    <div
                      className='del'
                      onClick={(e) => {
                        e.stopPropagation()
                        delRange(index)
                      }}
                    >
                      {i18n.t('businessHour.delPeriod')}
                    </div>
                  </div>
                  <div
                    className={`${_ns}-item`}
                    onClick={() => {
                      editClick(index, true, item.startTime)
                    }}
                  >
                    <div className={`${_ns}-item-title`}>
                      {i18n.t('businessHour.startBusiness')}
                    </div>
                    <div
                      className={classNames(`${_ns}-item-time`, {
                        active: editIndex === index && isStart,
                      })}
                    >
                      {i18n.t(
                        `businessHour.${item.startTime < list[0].startTime ? 'tomorrow' : 'today'}`,
                        { time: item.startTime },
                      )}
                    </div>
                    <div
                      className={classNames({
                        active: editIndex === index && isStart,
                      })}
                    >
                      <Icon name='_wenzi-wenben-bianji' size={20} />
                    </div>
                  </div>
                  <div
                    className={`${_ns}-item`}
                    onClick={() => {
                      editClick(index, false, item.endTime)
                    }}
                  >
                    <div className={`${_ns}-item-title`}>
                      {index === list.length - 1
                        ? i18n.t('businessHour.closeTime')
                        : i18n.t('businessHour.restTime', { time: index + 1 })}
                    </div>
                    <div
                      className={classNames(`${_ns}-item-time`, {
                        active: editIndex === index && !isStart,
                      })}
                    >
                      {i18n.t(
                        `businessHour.${item.endTime < list[0].startTime ? 'tomorrow' : 'today'}`,
                        { time: item.endTime },
                      )}
                    </div>
                    <div
                      className={classNames({
                        active: editIndex === index && !isStart,
                      })}
                    >
                      <Icon name='_wenzi-wenben-bianji' size={20} />
                    </div>
                  </div>
                  {index !== list.length - 1 && <div className='divider' />}
                </div>
              ))}
              <div className='add-time-btn' onClick={addRange}>
                {i18n.t('businessHour.addTime')}
              </div>
            </div>
            <div className='business-footer'>
              <DtButton
                className='btn-default'
                onClick={() => {
                  setIsEdit(false)
                  onConfirm(list)
                }}
              >
                {' '}
                {i18n.t('common.ok')}
              </DtButton>
              <div
                onClick={(e) => {
                  e.stopPropagation()
                  setIsEdit(false)
                  setList([...dataList])
                }}
              >
                {i18n.t('common.cancel')}
              </div>
            </div>
          </div>
        ) : (
          <div>
            {list.map((item, index) => (
              <div className='period-item' key={index}>
                <div className='left'>
                  {i18n.t(
                    `businessHour.${item.startTime < list[index].startTime ? 'tomorrow' : 'today'}`,
                    { time: item.startTime },
                  )}
                </div>
                <div className='center'>~</div>
                <div className='right'>
                  {i18n.t(
                    `businessHour.${item.endTime < list[index].startTime ? 'tomorrow' : 'today'}`,
                    { time: item.endTime },
                  )}
                </div>
              </div>
            ))}
          </div>
        )}
      </div>
      <TimeSelectModal
        isOpen={timeSelectVisible}
        value={editVal}
        onCancel={() => {
          setTimeSelectVisible(false)
          setEditIndex(undefined)
          setEditVal(undefined)
        }}
        onOk={(val) => {
          setTimeSelectVisible(false)
          setEditIndex(undefined)
          timeChange(val)
        }}
      />
    </>
  )
}

export default BusinessTime
