import { makeAutoObservable, runInAction } from 'mobx'
import { localStore as store } from '@dt/fe-base'
import storeMobx from './store.mobx'
import appMobx from './app.mobx'
import { query, mutate, newQuery } from '@dt/fe-business/client'
import { ListBrands, HQSelectBrand } from '@/gqlDocuments/brand'
import { GetUserBrandStoreSelection } from '@/gqlDocuments/newBrand'
import { ListBrandWithStoreCnt } from '@/gqlDocumentsNew/brand'
import { BrandBrand } from '@/gql/graphql'
import i18n from '@dt/fe-base/dist/i18n'
import { pick } from '@dt/fe-base/dist/lodash'
import { PermissionSdk } from '@dt/fe-business'
import userPermissionMobx from './userPermission.mobx'
import { BrandApi } from '@api'
import { App, CapacitorCookies, NoticePush, Storage } from '@dt/fe-capacitor'
import { LiveUpdate } from 'capacitor-plugin-live-update'
import { Capacitor } from '@capacitor/core'
import Cookies from 'js-cookie'
import { getHistoryNew } from '@common/hooks'

export const setBrandId = async (brandId: number) => {
  if (App.isNativePlatform()) {
    try {
      let jPushId = window?.Config?.jclientId
      // 如果 jclientId 不存在，则初始化极光
      if (!jPushId) {
        await NoticePush.init()
        const JPushClientId = await NoticePush.getClientId()
        window.Config.jclientId = JPushClientId
        jPushId = JPushClientId
      }
      userPermissionMobx.saveUserIdAndJPushId(
        store.get('ls.userInfo')?.userId,
        jPushId,
      )
    } catch (e) {
      console.log('消息初始化失败 ====>', e)
    }
    Cookies.remove('brandid')
    Cookies.set('brandid', `${brandId}`, { expires: 30 })
    CapacitorCookies.clearAllCookies()
    CapacitorCookies.setCookie({
      key: 'brandid',
      value: brandId.toString(),
      url: location.origin,
    })
    Storage.set('brandId', brandId.toString())
    Capacitor.isPluginAvailable('LiveUpdate') &&
      LiveUpdate.syncBundle({ brandId: String(brandId) })
  } else {
    Cookies.remove('brandid')
    Cookies.set('brandid', `${brandId}`, { expires: 30 })
  }
}

class BrandMobx {
  constructor() {
    makeAutoObservable(this)
  }
  brandList: BrandBrand[] = []
  brandInfo = {} as BrandBrand
  brandEdit: boolean = false
  currentBrandId: undefined | number = undefined
  brandTitle: string = i18n.t('store.plChooseBrand')

  oldPosNoBrandModal = false

  clear() {
    this.brandList = []
    this.brandInfo = {} as BrandBrand
    this.brandEdit = false
    this.brandTitle = i18n.t('store.plChooseBrand')
    this.oldPosNoBrandModal = false
    this.currentBrandId = undefined
  }

  async getBrandList(
    init: boolean,
    params?: { brandId?: number; notRefresh?: boolean },
  ) {
    const res: any = await newQuery(ListBrandWithStoreCnt)
    const { errors, data } = res
    if (!errors) {
      // 过滤老品牌
      const list = (data?.ListBrandWithStoreCnt?.list ?? []).filter(
        (b: any) => b.dataSource !== 'oldPos',
      )

      if (list?.length > 0) {
        runInAction(() => {
          this.brandList = list as BrandBrand[]
        })
        if (!init) {
          const obj = data?.ListBrandWithStoreCnt?.list?.find(
            (i: any) => i?.brandId === params?.brandId,
          )

          if (obj) {
            runInAction(() => {
              this.setBrandInfo(
                obj as BrandBrand,
                params?.notRefresh || false,
                false,
              )
            })
          }
        } else {
          let brandTitle = ''
          let brandInfo: BrandBrand | undefined
          if (list.length > 1) {
            brandTitle = i18n.t('store.plChooseBrand')
          } else {
            try {
              brandTitle = list[0].brandName || ''
              brandInfo = list[0] as BrandBrand
              await this.switchBrand({
                newBrandId: list[0].brandId,
                noCheck: true,
              })
            } catch (error) {}
          }

          runInAction(() => {
            this.setBrandTitle(brandTitle)
            brandInfo &&
              this.setBrandInfo(brandInfo, params?.notRefresh || false, false)
          })
        }
      } else {
        location.hash = '/brand'
      }
    }
    return false
  }

  async justUpdateList(filterOldBrand: boolean = true) {
    const { data, errors } = await query(ListBrands)
    if (!errors) {
      let list = data?.ListBrands ?? []
      if (filterOldBrand) {
        list = list.filter((b: any) => b.dataSource !== 'oldPos')
      }
      runInAction(() => {
        this.brandList = list as BrandBrand[]
      })
      return true
    }
    return false
  }

  async getPrevLoginBrandRecordNew(newBrandId: number, newStoreId?: number) {
    const brandInfo = store.get('ls.brandInfo')
    if (!brandInfo?.brandId) {
      store.set('ls.brandInfo', { brandId: newBrandId })
    }
    appMobx.setPageLoading(true)
    try {
      if (typeof newStoreId === 'undefined') {
        const { data, errors } = await newQuery<
          {
            GetUserBrandStoreSelectionV3: { storeId: number; brandId: number }
          },
          { userId: string; brandId: number }
        >(GetUserBrandStoreSelection, {
          userId: store.get('ls.userInfo')?.userId || '',
          brandId: newBrandId || store.get('ls.brandInfo')?.brandId,
        })

        if (!errors) {
          const { storeId } = data.GetUserBrandStoreSelectionV3
          return storeId
        }
      } else {
        // 登录后直接调用走这里
        await Promise.all([
          this.switchBrand({
            newBrandId: newBrandId!,
            notRefresh: true,
          }),
          storeMobx.justUpdateList(),
        ])

        let loginStoreId = newStoreId

        if (!loginStoreId) {
          loginStoreId =
            storeMobx.storeList.find((i) => i.status === 'enable')?.storeId ||
            storeMobx.storeList[0].storeId
        }
        const res = await storeMobx.setCurrentStore(loginStoreId)

        if (res) {
          appMobx.setPageLoading(false)
          store.set('ls.isSelectedBrand', 1)
          const history = getHistoryNew()

          const jumpTo = (path: string) => {
            if (history) {
              console.log('jumpTo', path)
              history.replace(path)
            } else {
              location.hash = path
            }
          }
          if (this.brandInfo.dataSource === 'oldPos') {
            if (PermissionSdk.checkModuleKey('newSupply')) {
              jumpTo('/home')
            } else {
              jumpTo('/storeDetail')
            }
          } else {
            jumpTo('/home')
          }
          this.reload()
          return true
        }
      }

      return false
    } catch (error) {
      appMobx.setPageLoading(false)
      return false
    }
  }

  async switchBrand(params: {
    newBrandId: number
    token?: string
    notRefresh?: boolean
    noCheck?: boolean
  }) {
    const { noCheck = false } = params
    if (params.newBrandId) {
      await setBrandId(params.newBrandId)
      const res = await mutate(HQSelectBrand, {
        ...params,
      })

      if (!res.errors) {
        const authorization = store.get('ls.authorization')
        // 兼容
        store.set('ls.authorization', {
          ...authorization,
          ...pick(res?.data?.HQSelectBrand, [
            'accessToken',
            'refreshToken',
            'accessExpire',
            'mqToken',
          ]),
        })
        if (res.data?.HQSelectBrand.accessToken) {
          PermissionSdk.clearAll(true)
          if (noCheck) {
            PermissionSdk.permissionInfoUpdate({
              token: res.data.HQSelectBrand.accessToken,
              brandId: params.newBrandId,
            })
            store.set('ls.isSelectedBrand', 1)
            await Promise.all([
              this.getBrandList(false, {
                brandId: params.newBrandId,
                notRefresh: params.notRefresh,
              }),
              userPermissionMobx.getUserInfo(params.newBrandId),
            ])
            return true
          } else {
            await PermissionSdk.permissionInfoUpdate({
              token: res.data.HQSelectBrand.accessToken,
              brandId: params.newBrandId,
            })
            if (PermissionSdk.hasLoginPerm()) {
              store.set('ls.isSelectedBrand', 1)
              await Promise.all([
                this.getBrandList(false, {
                  brandId: params.newBrandId,
                  notRefresh: params.notRefresh,
                }),
                userPermissionMobx.getUserInfo(params.newBrandId),
              ])
            } else {
              userPermissionMobx.updateNotLoginPermissionModal(true)
            }
            return PermissionSdk.hasLoginPerm()
          }
        }
      }

      return false
    }
  }

  async setBrandInfo(
    brandInfo: BrandBrand,
    notRefreshStore: boolean = false,
    isReload: boolean = true,
  ) {
    window.brandInfo = brandInfo
    window.Config.brandId = brandInfo.brandId as number
    store.set('ls.brandInfo', brandInfo)
    setBrandId(brandInfo.brandId)
    this.currentBrandId = brandInfo.brandId
    this.brandInfo = brandInfo

    if (!notRefreshStore) {
      setTimeout(() => {
        storeMobx.getStoreList(true)
      }, 0)
    }
    this.sendFinishUpdateBrand()
  }

  sendFinishUpdateBrand() {
    window.__eventBus.emit(
      window?.__eventBus?.eventEnumMap?.FinishUpdateBrand,
      JSON.parse(JSON.stringify(this.brandInfo || {})),
    )
  }

  setBrandEdit(status: boolean) {
    this.brandEdit = status
  }

  setBrandTitle(title: string) {
    this.brandTitle = title
  }

  updateOldPosNoBrandModal(flag: boolean) {
    this.oldPosNoBrandModal = flag
  }

  setCurrentBrandId(id: number) {
    this.currentBrandId = id
  }

  // 红蓝刷新触发接口
  async reload() {
    const brandId = store.get('ls.brandInfo')?.brandId
    brandId && setBrandId(brandId)
    const version = store.get('ls.bgVersion')
    try {
      const res: any = await BrandApi.getBrandVersion(brandId)
      if (res) {
        const getVersion = res.version || ''
        store.set('ls.bgVersion', getVersion)
        if ((!version && !getVersion) || version === getVersion) {
          return
        }
      }
    } catch (error) {
      console.error(error)
      return
    }
    setTimeout(() => {
      !App.isNativePlatform() && location.reload()
    }, 200)
  }
}

export default new BrandMobx()
