import { makeAutoObservable, runInAction } from 'mobx'
import { localStore } from '@dt/fe-base'
import { keyBy, sortBy } from '@dt/fe-base/dist/lodash'
import { GuideApi } from '@api'
import storeMobx from './store.mobx'
import brandMobx from './brand.mobx'

export enum ETodoType {
  // 创建品牌
  TodoTypeCreateBrand = 1,
  // 创建门店
  TodoTypeCreateStore,
  TodoSubTypeCreateStoreHaveStore,
  // POS点单
  TodoTypePosOrderingConfig,
  // 有一个POS订单
  TodoSubTypePosOrderingConfigHaveOrder,
  // POS 点单模式
  TodoSubTypePosOrderingConfigPattern,
  // 餐品库
  TodoSubTypePosOrderingConfigProduct,
  // pos 支付方式
  TodoSubTypePosOrderingConfigPayType,
  // App 下载和绑定
  TodoSubTypePosOrderingConfigApp,
  // 饿了么外卖
  TodoTypeElmConfig,
  // 饿了么授权完成
  TodoSubTypeElmConfigFinished,
  // 美团外卖
  TodoTypeMeiTuanConfig,
  // 美团外卖授权完成
  TodoSubTypeMeiTuanConfigFinished,
  // 员工和权限
  TodoTypeEmployee,
  // 员工和权限有一个员工
  TodoSubTypeEmployeeEmpManage,
  // 员工和权限角色配置
  TodoSubTypeEmployeeViewRole,
  // 饿了么关联餐品
  TodoSubTypeElemeCategoryMap,
  // 饿了么外卖预约单打印配置
  TodoSubTypeElemePrinterConfig,
  // 美团关联餐品
  TodoSubTypeMeituanCategoryMap,
  // 美团外卖预约单打印配置
  TodoSubTypeMeituanPrinterConfig,
}

/** todoType/subType 对应的页面 */
export const todoPathMap: Record<number, string> = {
  // 创建品牌
  [ETodoType.TodoTypeCreateBrand]: '/brand',
  // 创建门店
  [ETodoType.TodoTypeCreateStore]: '/store',
  [ETodoType.TodoSubTypeCreateStoreHaveStore]: '/store',
  // POS点单
  [ETodoType.TodoTypePosOrderingConfig]: '/mainline/orderMode',
  // POS 点单模式
  [ETodoType.TodoSubTypePosOrderingConfigPattern]: '/mainline/orderMode',
  // 餐品库
  [ETodoType.TodoSubTypePosOrderingConfigProduct]: '/menu/product',
  // pos 支付方式
  [ETodoType.TodoSubTypePosOrderingConfigPayType]: '/mainline/payment',
  // App 下载和绑定
  [ETodoType.TodoSubTypePosOrderingConfigApp]: '/mainline/guideDevice',
  // 员工和权限
  [ETodoType.TodoTypeEmployee]: '/user/user',
  // 员工和权限有一个员工
  [ETodoType.TodoSubTypeEmployeeEmpManage]: '/user/staff',
  // 员工和权限角色配置
  [ETodoType.TodoSubTypeEmployeeViewRole]: '/user/role',
  /** 有一个订单 */
  [ETodoType.TodoSubTypePosOrderingConfigHaveOrder]: '/orderSuccess',
  /** 饿了么外卖开通 */
  [ETodoType.TodoSubTypeElmConfigFinished]: '/mainline/takeoutAuth/eleme',
  /** 饿了么外卖关联餐品 */
  [ETodoType.TodoSubTypeElemeCategoryMap]:
    '/menu/takeout/productRelation/eleme',
  /** 饿了么外卖预约单打印配置 */
  [ETodoType.TodoSubTypeElemePrinterConfig]: '/mainline/takeout/printConfig',
  /** 美团外卖开通 */
  [ETodoType.TodoSubTypeMeiTuanConfigFinished]: '/mainline/takeoutAuth/meituan',
  /** 美团外卖关联餐品 */
  [ETodoType.TodoSubTypeMeituanCategoryMap]:
    '/menu/takeout/productRelation/meituan',
  /** 美团卖预约单打印配置 */
  [ETodoType.TodoSubTypeMeituanPrinterConfig]: '/mainline/takeout/printConfig',
}

/** 二次确认 */
export const checkAlertMap: Record<number, Record<string, string>> = {
  [ETodoType.TodoSubTypePosOrderingConfigProduct]: {
    // 默认餐品库警告
    default: 'productRequired',
    reward: 'productStepRequired',
  },
  [ETodoType.TodoSubTypePosOrderingConfigPayType]: {
    // 支付进件任务
    reward: 'paymentStepRequired',
  },
  [ETodoType.TodoSubTypeEmployeeEmpManage]: {
    // 员工数量
    reward: 'userStepRequired',
  },
}

class GuideMobx {
  /** 是否已完成所有开店步骤 */
  isFinishTodo = false
  todoList: IToDoList[] = []
  todoType: number = 0
  subType: number = 0
  /** 当前步骤 */
  step: number = 0
  /** 当前进行中的奖励 */
  reward: number = 0
  constructor() {
    makeAutoObservable(this)
    const storeInfo =
      (window as any).storeInfo || localStore.get('ls.storeInfo')
    this.isFinishTodo = localStore.get(
      `init.isFinishTodo.${storeInfo?.storeId}`,
    )
  }

  isInit: boolean = false
  /** 单次todo */
  get todoData() {
    return this.todoList.find((item) => item.todoType === this.todoType)
  }
  get checkList() {
    return this.todoData?.checkList || []
  }
  get checkData() {
    return this.checkList.find((item) => item.subType === this.subType)
  }

  get todoTypeToMap() {
    return keyBy(this.todoList, 'todoType')
  }

  get todoCount() {
    const total = this.todoList.length || 0
    const isFinish = this.todoList.filter((item) => item.finished).length || 0
    const unFinish = total - isFinish
    return {
      // 总步骤
      total,
      // 已完成的步骤
      isFinish,
      // 未完成的步骤
      unFinish,
      // 未完成的必填步骤
      unFinishRequired: this.todoList.filter(
        (item) => !item.finished && item.required,
      ).length,
    }
  }

  get subTypeToMap() {
    return keyBy(this.checkList, 'subType')
  }

  getCheckData = (todoType: number, subType: number) => {
    return this.todoTypeToMap[todoType]?.checkList.find(
      (item) => item.subType === subType,
    )
  }

  /** 校验二次确认 */
  checkNextConfirm = (subType: number) => {
    const alertMap = checkAlertMap[subType]
    if (!alertMap) {
      return false
    }
    console.log(this.checkData)
    if (!this.checkData?.checkPass && alertMap.default) {
      return {
        type: 'default',
        content: alertMap.default,
      }
    } else if (
      this.checkData?.checkPass &&
      this.checkData?.rewards?.length &&
      alertMap.reward
    ) {
      const reward = this.checkData.rewards.find((item) => !item.provide)
      if (reward) {
        return {
          type: 'reward',
          content: alertMap.reward,
          option: { num: reward?.number, day: reward?.storeTryDays || 5 },
        }
      }
    }
    return false
  }

  /** 校验是否可以下一步 */
  checkToNext = () => {
    const checkData = this.checkData
    if (!checkData) {
      return false
    }
    if (checkData.checkPass) {
      return true
    }
    if (checkData.steps) {
      return this.step < checkData.steps.length
    }
    if (checkData.rewards) {
      return checkData.rewards.some((item) => item.provide)
    }
    return false
  }

  setStep = (step: number) => {
    this.step = step
  }
  setReward = (reward: number) => {
    this.reward = reward
  }
  setTodoType = (todoType: number) => {
    this.todoType = todoType
  }
  setSubType = (subType: number) => {
    this.subType = subType
  }
  getList = async (isSync?: boolean) => {
    const storeInfo =
      (window as any).storeInfo || localStore.get('ls.storeInfo')
    const storeId = storeMobx.currentStoreId || storeInfo?.storeId
    const isFinish = localStore.get(`init.isFinishTodo.${storeId}`)
    this.isFinishTodo = isFinish
    if (isFinish && !isSync) return
    try {
      const data = await GuideApi.getTodoList({
        storeId,
      })
      runInAction(() => {
        const hasUnFinished = data.some((item) => !item.finished)
        if (!hasUnFinished) {
          localStore.set(`init.isFinishTodo.${storeId}`, true)
        }
        this.isInit = true
        this.todoList = sortBy((data as unknown as IToDoList[]) || [], 'id')
      })
    } catch (error) {}
    return
  }
}

export default new GuideMobx()
