import { makeAutoObservable } from 'mobx'
import i18n from '@dt/fe-base/dist/i18n'
import {
  home2,
  cutlery,
  role,
  board,
  supply,
  printer,
  calendar3,
  packages,
  bbq,
  shop,
  report,
  graphUp,
  message,
} from '@dt/ui-ionic-icon'
import { PermissionSdk } from '@dt/fe-business'

export interface IMenuItem {
  groupKey?: string
  sort: number
  path: string
  label?: () => string
  icon?: string
}

type TGroup = {
  label?: string
  menu: IMenuItem[]
  key: string
}

export const menuRouteConfig: TGroup[] = [
  {
    label: '',
    menu: [
      {
        sort: 1,
        path: '/home',
        label: () => i18n.t('menu.home'),
        icon: home2,
      },
    ],
    key: 'home',
  },
]

export const chatConfig: TGroup[] = [
  {
    label: '',
    menu: [
      {
        sort: 1,
        path: '/chat',
        label: () => i18n.t('menu.home'),
        icon: home2,
      },
    ],
    key: 'chat',
  },
]

const permissionLeftMenu = {
  '/mainline/home': false,
  '/storeDetail': true,
  '/mainline/device': false,
  '/menu/product': false,
  '/mainline/kitchen': false,
  'hq.employeeAccount': false,
  'hq.role': false,
  '/report/home': true,
  [PermissionSdk.PERM_KEY_SUPPLY.ORDER_GOODS_VIEW]: false,
  [PermissionSdk.PERM_KEY_SUPPLY.GOODS_VIEW]: false,
  [PermissionSdk.PERM_KEY_SUPPLY.SUPPLIER_VIEW]: false,
  [PermissionSdk.PERM_KEY_SUPPLY.REPORT_VIEW]: false,
}

export const permissionMenuRouteConfigMap: Record<string, IMenuItem> = {
  '/menu/product': {
    sort: 3,
    path: '/menu/product',
    label: () => i18n.t('menu.product'),
    icon: cutlery,
    groupKey: 'order',
  },
  '/mainline/home': {
    sort: 4,
    path: '/mainline/home',
    label: () => i18n.t('menu.mainline'),
    icon: board,
    groupKey: 'order',
  },
  '/mainline/kitchen': {
    sort: 5,
    path: '/mainline/kitchen',
    label: () => i18n.t('menu.kitchen'),
    icon: bbq,
    groupKey: 'order',
  },
  '/storeDetail': {
    sort: 6,
    path: '/storeDetail',
    label: () => i18n.t('menu.storeInfo'),
    icon: shop,
    groupKey: 'store',
  },
  '/mainline/device': {
    sort: 7,
    path: '/mainline/device',
    label: () => i18n.t('menu.device'),
    icon: printer,
    groupKey: 'store',
  },
  'hq.employeeAccount': {
    sort: 8,
    path: '/user/user/staff',
    label: () => i18n.t('menu.staffSetting'),
    icon: role,
    groupKey: 'staff',
  },
  'hq.role': {
    sort: 9,
    path: '/user/user/role',
    label: () => i18n.t('menu.staffRoleManage'),
    icon: role,
    groupKey: 'staff',
  },
  '/report/home': {
    sort: 10,
    path: '/report/home',
    label: () => i18n.t('menu.revenueReport'),
    icon: report,
    groupKey: 'report',
  },
  [PermissionSdk.PERM_KEY_SUPPLY.ORDER_GOODS_VIEW]: {
    sort: 11,
    path: PermissionSdk.PERM_KEY_SUPPLY.ORDER_GOODS,
    label: () => i18n.t('menu.callGoodsCalendar'),
    icon: calendar3,
    groupKey: 'callGoods',
  },
  [PermissionSdk.PERM_KEY_SUPPLY.GOODS_VIEW]: {
    sort: 12,
    path: PermissionSdk.PERM_KEY_SUPPLY.GOODS,
    label: () => i18n.t('menu.goods'),
    icon: packages,
    groupKey: 'callGoods',
  },
  [PermissionSdk.PERM_KEY_SUPPLY.SUPPLIER_VIEW]: {
    sort: 13,
    path: PermissionSdk.PERM_KEY_SUPPLY.SUPPLIER,
    label: () => i18n.t('menu.supplier'),
    icon: supply,
    groupKey: 'callGoods',
  },
  [PermissionSdk.PERM_KEY_SUPPLY.REPORT_VIEW]: {
    sort: 14,
    path: PermissionSdk.PERM_KEY_SUPPLY.REPORT,
    label: () => i18n.t('menu.orderReport'),
    icon: graphUp,
    groupKey: 'report',
  },
}

class RouterManger {
  layoutMenu: TGroup[] = []

  constructor() {
    makeAutoObservable(this)
    window?.__eventBus.on(
      window?.__eventBus?.eventEnumMap?.modulePermissionListUpdate,
      () => {
        this.updateFinalLayoutMenuList()
      },
    )
    window?.__eventBus.on(
      window?.__eventBus?.eventEnumMap?.permissionInfoUpdate,
      () => {
        this.updateFinalLayoutMenuList()
      },
    )

    window?.__eventBus.on(
      window?.__eventBus?.eventEnumMap?.permissionSdkClear,
      () => {
        this.updateFinalLayoutMenuList()
      },
    )
  }

  clear() {
    this.layoutMenu = []
  }

  async updateFinalLayoutMenuList() {
    const newPermissionList = await PermissionSdk.check(
      Object.keys(permissionLeftMenu),
    )

    Object.keys(newPermissionList).forEach((routeKey) => {
      if (!newPermissionList[routeKey]) return
      switch (routeKey) {
        case PermissionSdk.PERM_KEY_SUPPLY.ORDER_GOODS_VIEW:
        case PermissionSdk.PERM_KEY_SUPPLY.GOODS_VIEW:
        case PermissionSdk.PERM_KEY_SUPPLY.SUPPLIER_VIEW:
        case PermissionSdk.PERM_KEY_SUPPLY.REPORT_VIEW: {
          !PermissionSdk.checkModuleKey('newSupply') &&
            (newPermissionList[routeKey] = false)
          break
        }
        case '/user/user':
        case '/mainline/home':
        case '/mainline/device':
        case '/menu/product':
        case '/mainline/kitchen': {
          !PermissionSdk.checkModulePermissionKey(routeKey) &&
            (newPermissionList[routeKey] = false)
          break
        }
      }
    })

    let list: IMenuItem[] = []

    Object.keys(newPermissionList).forEach((key) => {
      if (!newPermissionList[key]) return
      list.push(permissionMenuRouteConfigMap[key])
    })
    // 排序
    list = list.sort((a, b) => a.sort - b.sort)

    // console.log(list, '=====role list')
    // 分组
    const groupList = list.reduce(
      (prev, curr) => {
        const currentGroup = prev.find((i) => i.key === curr.groupKey)
        currentGroup && currentGroup.menu.push(curr)
        return prev
      },
      [
        { label: 'order', menu: [], key: 'order' },
        { label: 'store', menu: [], key: 'store' },
        { label: 'staff', menu: [], key: 'staff' },
        { label: 'report', menu: [], key: 'report' },
        { label: 'callGoods', menu: [], key: 'callGoods' },
      ] as TGroup[],
    )

    // 添加客服分组
    // groupList = chatConfig.concat(groupList)
    this.layoutMenu = menuRouteConfig.concat(groupList)
    // this.layoutMenu = list
  }
}

export default new RouterManger()
