import { makeAutoObservable, runInAction } from 'mobx'
import { forEach, keyBy, map } from '@dt/fe-base/dist/lodash'
import { EEventNames } from '@common/eventBus/enum'
import { GuideApi } from '@api'
import { EDtTaskType } from '@dt/ui-mobile-business'
import { PermissionSdk } from '@dt/fe-business'
import i18n from '@dt/fe-base/dist/i18n'
import { getHistoryNew } from '@common/hooks'

export enum ENextType {
  CHILD = 'touchChild',
  MASK = 'touchMask',
}

export enum ETaskStatus {
  /** 未完成 */
  CANCEL = 'cancel',
  /** 已完成 */
  FINISHED = 'finish',
}

const taskTypeToPermissionType = {
  [EDtTaskType.OPEN_PRODUCT]: '/menu/product',
  [EDtTaskType.OPEN_POS_DEVICE]: 'hq.posOrderDevice',
  [EDtTaskType.OPEN_PRINTER]: 'hq.printer',
  [EDtTaskType.OPEN_POS_ORDER]: '/mainline/home',
  [EDtTaskType.OPEN_WECHAT_ALIPAY]: 'hq.posPaySettings',
  [EDtTaskType.OPEN_USER]: '/user/user',
  [EDtTaskType.OPEN_TAKEOUT_ELEME]: '/mainline/takeoutAuth/eleme',
  [EDtTaskType.OPEN_TAKEOUT_MEITUAN]: '/mainline/takeoutAuth/meituan',
}

/** 将taskType转成permissionType */
const permissionTypeToTaskType = () => {
  const res: Record<string, string> = {}
  forEach(taskTypeToPermissionType, (value, key) => {
    res[value] = key
  })
  return res
}

/** 校验授权包 */
export const checkAuth = (taskType: EDtTaskType) => {
  switch (taskType) {
    case EDtTaskType.OPEN_POS_ORDER:
      return PermissionSdk.checkModuleKey('posOrder')
    case EDtTaskType.OPEN_TAKEOUT_ELEME:
    case EDtTaskType.OPEN_TAKEOUT_MEITUAN:
      return PermissionSdk.checkModuleKey('threePartyTakeout')
    /** 其他授权，只要有其中一个就行 */
    case EDtTaskType.OPEN_PRODUCT:
    case EDtTaskType.OPEN_POS_DEVICE:
    case EDtTaskType.OPEN_PRINTER:
    case EDtTaskType.OPEN_WECHAT_ALIPAY:
      return (
        PermissionSdk.checkModuleKey('posOrder') ||
        PermissionSdk.checkModuleKey('threePartyTakeout')
      )
    default:
      return true
  }
}

/** 校验权限 */
export const checkPermission = async (taskTypes: { type: EDtTaskType }[]) => {
  const res: Record<string, boolean> = await PermissionSdk.check(
    map(taskTypes, (task) => taskTypeToPermissionType[task.type]),
  )
  const typeMap = permissionTypeToTaskType()
  /** 转成EDtTaskType */
  const resTypes: Record<string, boolean> = {}
  forEach(res, (value, key) => {
    resTypes[typeMap[key as keyof typeof typeMap] as unknown as string] = value
  })
  return resTypes
}

class TaskMobx {
  constructor() {
    makeAutoObservable(this)
    enum ETargetType {
      SUCCESS = 'success',
      CLOSE = 'close',
      UPDATE = 'update',
    }
    // 监听任务更新
    window.__eventBus?.on(
      EEventNames.TaskUpdate,
      (data: { taskType: EDtTaskType | null; targetType: ETargetType }) => {
        console.log('任务更新', data)
        this.setTargetTask()
        /** 任务完成 */
        if (data.targetType === ETargetType.SUCCESS) {
          this.finishTask({
            isCancel: false,
            taskType: data.taskType as string,
          })
        } else if (data.targetType === ETargetType.CLOSE) {
          const history = getHistoryNew()
          /** 关闭任务，需要进入任务列表 */
          history.replace('/task')
        }
      },
    )
  }
  /** 任务列表 */
  taskList: ITaskList[] = []
  /** 目标任务 */
  targetTask: EDtTaskType | undefined = undefined

  /** 完成任务数量 */
  finishedCnt: number = 0
  /** 未完成任务数量 */
  unfinishedCnt: number = 0

  /** 任务类型映射 */
  get taskTypeToMap() {
    return keyBy(this.taskList, 'type')
  }

  /** 设置目标任务 */
  setTargetTask = (taskType?: EDtTaskType) => {
    this.targetTask = taskType
    window.runingTaskType = taskType
  }

  /** 获取任务列表 */
  getTaskList = async () => {
    const res: any[] = await GuideApi.getTaskList()
    const permissionMap = await checkPermission(res)
    runInAction(() => {
      let finishedCnt = 0
      let unfinishedCnt = 0
      this.taskList = map(res, (item) => {
        if (item.status === 'finish') {
          finishedCnt++
        } else {
          unfinishedCnt++
        }
        return {
          name: i18n.t(`task.name.${item.type}`),
          desc: i18n.t(`task.desc.${item.type}`),
          status: item.status || ETaskStatus.CANCEL,
          type: item.type,
          /** 是否授权 */
          isAuth: checkAuth(item.type),
          /** 是否有权限 */
          isPermission: permissionMap[item.type],
          basicTaskNo: item.basicTaskNo,
          isCanReverse: item.isCanReverse,
          isCanComplete: item.isCanComplete,
          isCanCancel: item.isCanCancel,
          sort: item.sort,
        }
      }).filter(Boolean) as ITaskList[]
      this.finishedCnt = finishedCnt
      this.unfinishedCnt = unfinishedCnt
    })
  }

  /** 完成任务 */
  finishTask = async ({
    isCancel = false,
    taskType,
  }: {
    isCancel: boolean
    taskType: string
  }) => {
    try {
      const res = await GuideApi.finishTask({ isCancel, taskType })
      if (res) {
        this.getTaskList()
      }
    } catch (error) {
      console.error(error)
    }
    return true
  }
}

export default new TaskMobx()
