import { makeAutoObservable } from 'mobx'
import { newQuery, newMutate } from '@dt/fe-business/client'
import { HqUserInfo, SaveUserIdAndJPushId } from '@/gqlDocuments/newUser'
import { Trace } from '@dt/fe-capacitor'
import { localStore, trace } from '@dt/fe-base'
import type { RolePermUserCenterPerm, HqUserInfoQuery } from '@/gql/graphql'
import { PermissionSdk } from '@dt/fe-business'
import posthog from 'posthog-js'

export interface IPermissionObj {
  isSuper: boolean
  isParentManger: boolean
  isStoreManger: boolean
  userRoleNameMap: Record<string, boolean>
  permissionIdMap: Record<string, Record<string, boolean>>
  permissionUrlMap: Record<string, string[]>
  moduleKeyList: string[]
}

export const defaultPermissionObj: IPermissionObj = {
  isSuper: false,
  isParentManger: false,
  isStoreManger: false,
  userRoleNameMap: {},
  permissionIdMap: {},
  permissionUrlMap: {},
  moduleKeyList: [],
}

class UserPermission {
  currentUserPermission: RolePermUserCenterPerm[] = []

  permissionObj: IPermissionObj = JSON.parse(
    JSON.stringify(defaultPermissionObj),
  )

  moduleKeyList: string[] = []

  userInfo: HqUserInfoQuery['HqUserInfo'] | null = null

  notLoginPermissionModal: boolean = false

  constructor() {
    makeAutoObservable(this)
  }

  async getUserInfo(brandId: number, isInit = false) {
    const { data, errors } = await newQuery<
      { HqUserInfoV3: { userInfo: IUserInfo } },
      { brandId: number; userId: string }
    >(HqUserInfo, {
      brandId,
      userId: localStore.get('ls.userInfo')?.userId as string,
    })
    if (!errors) {
      const userInfo = data?.HqUserInfoV3?.userInfo || {}
      window.userInfo = {
        ...localStore.get('ls.userInfo'),
        ...userInfo,
      }
      localStore.set('ls.userInfo', window.userInfo)
      this.userInfo = window.userInfo
      window.__eventBus?.emit?.(
        window.__eventBus?.eventEnumMap?.UserInfoUpdate,
        userInfo,
      )
      // 监控
      try {
        if ('employeeId' in userInfo && !isInit) {
          console.log('重置灰度')
          posthog.reset()
          posthog.identify(String(userInfo.employeeId), {
            ...userInfo,
            brandId,
          })
          posthog.reloadFeatureFlags()
          posthog.onFeatureFlags(() => {
            console.log('onFeatureFlags posthog.identify', userInfo)
          })
        }

        Trace?.setUser?.({
          id: `${Config.brandId || ''}_${Config.storeId || ''}_${Config.userId || ''}_${Config.deviceId || ''}`,
          username: this.userInfo?.userName,
        })
        trace?.setConfig?.({
          uid: `${Config.brandId || ''}_${Config.storeId || ''}_${Config.userId || ''}_${Config.deviceId || ''}`,
        })
        trace?.setUsername?.(this.userInfo?.userName || '')
      } catch (error) {}
    }
  }

  // 授权接口
  async modulePermissionUpdate(storeId: number) {
    await PermissionSdk.modulePermissionUpdate(storeId)
    if (
      !PermissionSdk.checkModuleKey('newSupply') &&
      location.hash.includes('/supply')
    ) {
      const deviceInfo = localStore.get('ls.deviceInfo')
      if (!deviceInfo?.deviceId) {
        location.hash = '#/store'
      }
    }
  }

  updateMobxCache(res: IPermissionObj, permData: RolePermUserCenterPerm[]) {
    this.clear()
    this.currentUserPermission = permData || []

    // 将当前用户权限数据写入全局
    if (permData?.[0]) {
      window.userInfo = {
        ...localStore.get('ls.userInfo'),
        ...(permData?.[0] || {}),
      }
      localStore.set('ls.userInfo', window.userInfo)
    }
    this.permissionObj = res
    window.permissionInfo = this.permissionObj
    const storeId = String(window.Config.storeId || 0)
    this.changeRoutePermByStore(storeId)
    window.__eventBus.emit(
      window?.__eventBus?.eventEnumMap?.UpdateUserPermission,
      this.permissionObj,
    )
  }

  updatePermissionObj(data: RolePermUserCenterPerm[]) {
    const resObj: IPermissionObj = JSON.parse(
      JSON.stringify(defaultPermissionObj),
    )

    resObj.moduleKeyList = this.moduleKeyList

    let hasLoginPerm = false

    data.forEach((e) => {
      resObj.userRoleNameMap[e.roleName!] = true
      let [prev, role] = e.roleKey!.split('_')
      if (!role) {
        role = prev
      }
      if (e.isSuper || role === 'owenr') {
        resObj.isSuper = true
      }
      if (role === 'operationalManager') {
        resObj.isParentManger = true
      }
      if (role === 'storeManager') {
        resObj.isStoreManger = true
      }

      if (!hasLoginPerm) {
        const res = e.rolePermList.find((v) => v.url === 'hq.login')

        res && (hasLoginPerm = true)
      }

      this.setPermissionIdObj(resObj, e.storeIds, e.rolePermList)
    })

    return {
      res: resObj,
      hasLoginPerm,
    }
  }

  changeRoutePermByStore(storeId?: string) {
    let currentStorePermissionUrlList: string[]
    if (!storeId) {
      this.moduleKeyList = []
      currentStorePermissionUrlList = []
    } else {
      currentStorePermissionUrlList =
        this.permissionObj.permissionUrlMap[storeId] || []
    }
  }

  clear(shouldNotice = false) {
    this.moduleKeyList.length = 0
    this.currentUserPermission.length = 0
    this.permissionObj = JSON.parse(JSON.stringify(defaultPermissionObj))
    window.permissionInfo = this.permissionObj

    if (shouldNotice) {
      window.__eventBus.emit(
        window?.__eventBus?.eventEnumMap?.UpdateUserPermission,
        this.permissionObj,
      )
    }
  }

  setPermissionIdObj(
    data: IPermissionObj,
    storeIds: RolePermUserCenterPerm['storeIds'],
    item: RolePermUserCenterPerm['rolePermList'],
  ) {
    const idMap: Record<string, boolean> = {}
    const urlList: string[] = []

    storeIds?.forEach((storeId, index) => {
      if (index === 0) {
        const oldObj = data.permissionIdMap[storeId] || {}
        const oldList = data.permissionUrlMap[storeId] || []
        this.setPermissionId(idMap, urlList, item)
        data.permissionIdMap[storeId] = { ...oldObj, ...idMap }
        data.permissionUrlMap[storeId] = Array.from(
          new Set(oldList.concat(urlList)),
        )
      } else {
        const oldObj = data.permissionIdMap[storeId] || {}
        const oldList = data.permissionUrlMap[storeId] || []
        data.permissionIdMap[storeId] = { ...oldObj, ...idMap }
        data.permissionUrlMap[storeId] = Array.from(
          new Set(oldList.concat(urlList)),
        )
      }
    })
  }

  setPermissionId(
    obj: Record<string, boolean>,
    list: string[],
    item: RolePermUserCenterPerm['rolePermList'],
  ) {
    item?.forEach((v) => {
      obj[v.id!] = true
      v.url && list.push(v.url)
      if (v.children) {
        this.setPermissionId(obj, list, v.children)
      }
    })
  }

  isHasPermission(permissionId: string) {
    const storeId: string = localStore.get('ls.storeInfo')?.storeId
    return !!this.permissionObj.permissionIdMap[storeId]?.[permissionId]
  }

  /**
   * 上传用户ID和极光ID
   * @param userId
   * @param jPushId
   * @returns
   */
  saveUserIdAndJPushId(userId?: string, jPushId?: string) {
    if (userId && jPushId) {
      return newMutate(SaveUserIdAndJPushId, {
        userId,
        jPushId,
        platform: 'hq',
      })
    }
  }

  updateNotLoginPermissionModal(flag: boolean) {
    this.notLoginPermissionModal = flag
  }
}

const userPermissionMobxInstance = new UserPermission()

UserPermission.prototype.isHasPermission.bind(userPermissionMobxInstance)

export default userPermissionMobxInstance
