import { CSSProperties, ReactNode, useEffect, useMemo, useRef } from 'react'
import { DndProvider, useDrag, useDrop, useDragLayer } from 'react-dnd'
import { TouchBackend } from 'react-dnd-touch-backend'
import { motion, AnimatePresence } from 'framer-motion'
import { classNames } from '@dt/fe-base'
import cardMobx from '@mobx/card.mobx'
import { observer } from 'mobx-react'
import translate from '@utils/translate'
import { plus } from '@dt/ui-ionic-icon'
import { DtIcon } from '@dt/ui-mobile'
import './index.scss'

const dragType = 'box'
type TDragItem = {
  name: string
}
interface IDragItemProps {
  className?: string
  name: string
  children?: ReactNode
  style?: CSSProperties
}
const DragItem = (props: IDragItemProps) => {
  const { name, className, style, children } = props
  const ref = useRef<HTMLDivElement>(null)

  const [{ isDragging, offset }, drag] = useDrag({
    type: dragType,
    item: { name },
    collect: (monitor) => ({
      isDragging: monitor.isDragging(),
      offset: monitor.getClientOffset(),
    }),
  })
  drag(ref)
  return (
    <>
      <motion.div
        className={classNames(className, { dragging: isDragging })}
        ref={ref}
        layout
        transition={{ duration: 0.3 }}
        style={style}
        data-card-name={name}
      >
        {children}
      </motion.div>
      {isDragging && offset && ref.current && (
        <div
          className={className}
          style={{
            position: 'fixed',
            pointerEvents: 'none',
            left: offset.x - (ref.current?.clientWidth || 0) / 2,
            top: offset.y - (ref.current?.clientHeight || 0) / 2,
            zIndex: 100,
          }}
          dangerouslySetInnerHTML={{ __html: ref.current?.innerHTML }}
        />
      )}
    </>
  )
}
interface IDropItemProps extends IDragItemProps {
  clear: () => void
  move: (props: { fromName: string; toName: string }) => void
}
const Blank = (props: IDropItemProps) => {
  const { move, clear, name, className, style } = props
  const ref = useRef<HTMLDivElement>(null)
  const [, drop] = useDrop({
    accept: dragType,
    hover: (item: TDragItem) => {
      move({ fromName: item.name, toName: name })
    },
    drop() {
      clear()
    },
  })
  drop(ref)
  useEffect(() => {
    const mappedElement = document.querySelector(`[data-card-name='${name}']`)
    if (mappedElement && !mappedElement.innerHTML && ref.current) {
      ref.current.style.display = 'none'
    }
  }, [])
  return (
    <div
      className={classNames(className, 'blank')}
      style={style}
      ref={ref}
    ></div>
  )
}
type TDndListProps = {
  isImg?: boolean
  showDelete: boolean
  setShowDelete: (val: boolean) => void
}
const DndList = observer((props: TDndListProps) => {
  const { showDelete, isImg, setShowDelete } = props
  const { isDragging } = useDragLayer((monitor) => ({
    isDragging: monitor.isDragging(),
  }))
  const { cardList } = cardMobx.group
  const movingRef = useRef({ fromName: '', toName: '' })

  const clear = () => {
    movingRef.current.fromName = ''
    movingRef.current.toName = ''
  }
  const move: IDropItemProps['move'] = (props) => {
    const { fromName, toName } = props
    if (
      fromName !== toName &&
      movingRef.current.fromName === fromName &&
      movingRef.current.toName === toName
    ) {
      return
    }
    movingRef.current.fromName = fromName
    movingRef.current.toName = toName
    cardMobx.moveCard(fromName, toName)
  }

  const blankList = useMemo(() => {
    const list = cardList.filter((item) => !item.hide)
    const length = Math.ceil(window.innerHeight / 121)
    if (list.length > length || list.length === 0) {
      return list
    }
    const lastItem = list[list.length - 1]
    return [
      ...list,
      ...new Array(length - list.length).fill('').map(() => lastItem),
    ]
  }, [cardList])

  return (
    <div className='dnd-container'>
      <div className='module-card-container' style={{ zIndex: 1 }}>
        <AnimatePresence>
          {cardList.map((item) => {
            const Component = item.component
            if (item.hide) {
              return null
            }
            return (
              <DragItem
                key={item.name}
                name={item.name}
                className={classNames(
                  `item-card-wrapper ${item.name} ${item.size}`,
                )}
                style={item.height ? { height: item.height } : undefined}
              >
                <Component
                  isImg={isImg}
                  item={item}
                  showDelete={showDelete}
                  disabledClick
                  onDelete={() => cardMobx.deleteCard(item.name)}
                  onLongPress={() => {
                    setShowDelete(true)
                  }}
                />
              </DragItem>
            )
          })}
        </AnimatePresence>
      </div>
      <div
        className='module-card-container blank-container'
        style={isDragging ? { zIndex: 2 } : undefined}
      >
        {blankList.map((item, i) => {
          return (
            <Blank
              key={i}
              name={item.name}
              style={item.height ? { height: item.height } : undefined}
              className={classNames(`item-card-wrapper ${item.size}`)}
              clear={clear}
              move={move}
            />
          )
        })}
      </div>
    </div>
  )
})

const EditCardList = (props: TDndListProps) => {
  if (cardMobx.group.cardList.length) {
    return (
      <DndProvider backend={TouchBackend}>
        <DndList {...props} />
      </DndProvider>
    )
  }
  return null
}

export default observer(EditCardList)
