import React, { useState } from 'react'
import { observer } from 'mobx-react'
import './index.scss'
import { DtIcon, DtRippleEffect, loadingLayer, toastLayer } from '@dt/ui-mobile'
import { chevronForwardOutline, checkmarkOutline } from 'ionicons/icons'
import taskMobx, { ETaskStatus } from '@mobx/task.mobx'
import { useHistory } from 'react-router'
import { EDtTaskType } from '@dt/ui-mobile-business'
import i18n from '@dt/fe-base/dist/i18n'

interface IProps {
  item: ITaskList
  from?: string
}

enum ETaskErrorTyps {
  /** 取消 */
  CANCEL = 'cancel',
  /** 完成 */
  FINISHED = 'finished',
}

const taskTypeToErrorTips = {
  [EDtTaskType.OPEN_PRODUCT]: {
    [ETaskErrorTyps.CANCEL]: 'task.error.openCancel',
    [ETaskErrorTyps.FINISHED]: 'task.error.openFinished',
  },
  [EDtTaskType.OPEN_POS_DEVICE]: {
    [ETaskErrorTyps.CANCEL]: 'task.error.openCancel',
    [ETaskErrorTyps.FINISHED]: 'task.error.openFinished',
  },
  [EDtTaskType.OPEN_PRINTER]: {
    [ETaskErrorTyps.CANCEL]: 'task.error.openCancel',
    [ETaskErrorTyps.FINISHED]: 'task.error.openFinished',
  },
  [EDtTaskType.OPEN_WECHAT_ALIPAY]: {
    [ETaskErrorTyps.CANCEL]: 'task.error.openCancel',
    [ETaskErrorTyps.FINISHED]: 'task.error.openFinished',
  },
  [EDtTaskType.OPEN_POS_ORDER]: {
    [ETaskErrorTyps.CANCEL]: 'task.error.posOrderCancel',
    [ETaskErrorTyps.FINISHED]: 'task.error.openFinished',
  },
  [EDtTaskType.OPEN_USER]: {
    [ETaskErrorTyps.CANCEL]: 'task.error.openCancel',
    [ETaskErrorTyps.FINISHED]: 'task.error.openFinished',
  },
  [EDtTaskType.OPEN_TAKEOUT_MEITUAN]: {
    [ETaskErrorTyps.CANCEL]: 'task.error.posMeituanCancel',
    [ETaskErrorTyps.FINISHED]: 'task.error.openFinished',
  },
  [EDtTaskType.OPEN_TAKEOUT_ELEME]: {
    [ETaskErrorTyps.CANCEL]: 'task.error.posElemeCancel',
    [ETaskErrorTyps.FINISHED]: 'task.error.openFinished',
  },
}

const TaskItem: React.FC<IProps> = ({ item, from }) => {
  const _ns = 'dt-mobile-main-task-item-container'
  const checked = item.status === ETaskStatus.FINISHED
  const history = useHistory()
  /** 不能重新开始任务的类型 */
  const cantStartTypes = [
    EDtTaskType.OPEN_POS_ORDER,
    EDtTaskType.OPEN_TAKEOUT_ELEME,
    EDtTaskType.OPEN_TAKEOUT_MEITUAN,
  ]

  const handleFinish = async () => {
    if (checkPermission()) {
      return
    }
    if (!checked && !item.isCanComplete) {
      toastLayer.error(
        i18n.t(taskTypeToErrorTips[item.type as EDtTaskType].finished, {
          name: item.name,
        }),
      )
      return
    }
    /** 已完成，且不能取消 */
    if (checked && !item.isCanCancel) {
      toastLayer.error(
        i18n.t(taskTypeToErrorTips[item.type as EDtTaskType].cancel, {
          name: item.name,
        }),
      )
      return
    }
    loadingLayer.show()
    await taskMobx.finishTask({ isCancel: checked, taskType: item.type })
    loadingLayer.hide()
  }
  const checkPermission = () => {
    /** 授权 */
    if (!item.isAuth) {
      toastLayer.error(i18n.t('task.error.noAuth'))
      return true
    }
    /** 权限 */
    if (!item.isPermission) {
      toastLayer.error(i18n.t('task.error.noPermission'))
      return true
    }
    return false
  }
  const handleClick = () => {
    if (checkPermission()) {
      return
    }
    /** 已完成，且不能重新开始 */
    if (checked && cantStartTypes.includes(item.type as EDtTaskType)) {
      return
    }
    history.replace('/home')
    taskMobx.setTargetTask(item.type as unknown as EDtTaskType)
  }

  return (
    <div
      className={`${_ns} ${checked ? 'checked' : ''} ripple-parent rectangle`}
    >
      <div className='task-item-checkbox' onClick={handleFinish}>
        <DtIcon icon={checkmarkOutline} />
      </div>
      <div className='task-item-content' onClick={handleClick}>
        <div className='task-item-title'>{item.name}</div>
        <div className='task-item-desc'>{item.desc}</div>
      </div>
      {(!checked || !cantStartTypes.includes(item.type as EDtTaskType)) && (
        <div className='task-path' onClick={handleClick}>
          <DtIcon icon={chevronForwardOutline} />
        </div>
      )}
      <DtRippleEffect />
    </div>
  )
}

export default observer(TaskItem)
