import { CapacitorHttp, HttpResponse } from '@dt/fe-capacitor/dist/Core'
import { App } from '@dt/fe-capacitor'
import { dayjs, localStore as store } from '@dt/fe-base'
import ENV from '@common/env'
import provider from '@init/provider'
import { clearStore } from '@utils'

interface IHttpRequestOptions {
  url: string
  method?: 'GET' | 'POST' | 'PUT' | 'DELETE'
  data?: any
  params?: any
  headers?: any
  /** 是否是刷新操作 */
  isRefresh?: boolean
  /** 读取超时时间 */
  readTimeout?: number
}

/** 刷新日期，用于长期未刷新的场景 */
let refreshDate: string = ''

/** 刷新token */
export function getRefreshToken() {
  const authorization: any = store.get('ls.authorization')
  const refreshToken = authorization?.accessToken || ''
  if (refreshToken && dayjs().format('YYYY-MM-DD') !== refreshDate) {
    return http({
      url: `${ENV.HQBFF}/app/refreshJwt`,
      data: {
        token: refreshToken,
      },
      method: 'POST',
      isRefresh: true,
    }).then((res: any) => {
      console.log(res, 'refreshJWT')
      // refreshToken 没有过期，重新获取token 并继续请求
      if (res.data && res.data.refreshToken) {
        const authorization = store.get('ls.authorization')
        store.set('ls.authorization', {
          ...authorization,
          ...(res?.data || {}),
        })
        refreshDate = dayjs().format('YYYY-MM-DD')
      }
      return authorization
    })
  }
  return authorization
}

async function http(
  requestOptions: IHttpRequestOptions,
): Promise<HttpResponse> {
  const brandId =
    store.get('ls.brandInfo')?.brandId || window.brandInfo?.brandId || 0
  // 在请求发送前执行的操作，比如设置统一的请求头
  const defaultHeaders: any = {
    'Content-Type': 'application/json',
    // 这里添加更多默认请求头
    brandid: brandId,
  }

  const authorization: any = store.get('ls.authorization')
  const sessionId: any = store.get('ls.sessionId')

  if (authorization?.accessToken) {
    defaultHeaders.Authorization = 'Bearer ' + authorization.accessToken
  }

  if (sessionId) {
    defaultHeaders.Dtdomain = sessionId
  }

  const webTracerWithZone = provider.getTracer('CapacitorHttp')
  const span = webTracerWithZone.startSpan('http-request-span', {})
  // 添加 B3 追踪头
  if (span) {
    const spanContext = span.spanContext()
    defaultHeaders['X-B3-TraceId'] = spanContext.traceId
    defaultHeaders['X-B3-SpanId'] = spanContext.spanId
    defaultHeaders['X-B3-Sampled'] = '1'
    defaultHeaders['X-B3-Flags'] = spanContext.traceFlags
  }

  // version
  const { main } = window.versionInfo || {}
  if (main?.appVersion) defaultHeaders['hq-version'] = main?.appVersion

  if (!requestOptions.method) {
    requestOptions.method = 'POST'
  }
  requestOptions.headers = { ...defaultHeaders, ...requestOptions.headers }
  /** get 方法拉平data方式 */
  if (requestOptions.method === 'GET' && requestOptions.data) {
    requestOptions.params = requestOptions.data
    delete requestOptions.data
  }

  /** 给接口添加brandId */
  if (brandId && !requestOptions?.headers?.noBrandId) {
    // 不需要brandId 在头上加 'noBrandId'
    if (requestOptions.method !== 'GET') {
      if (requestOptions.data) {
        requestOptions.data = {
          ...(requestOptions?.data || {}),
          brandId: Number(brandId),
        }
      } else {
        requestOptions.data = { brandId: Number(brandId) }
      }
    } else {
      if (requestOptions.params) {
        requestOptions.params.brandId = String(
          requestOptions.params.brandId || brandId,
        )
      } else {
        requestOptions.params = { brandId: String(brandId) }
      }
    }
  }
  try {
    // 发送 HTTP 请求
    const response = await CapacitorHttp.request({
      method: requestOptions.method || 'POST',
      url: requestOptions.url,
      headers: requestOptions.headers,
      data:
        App.getPlatform() === 'web'
          ? requestOptions.data
          : JSON.stringify(requestOptions.data),
      params: requestOptions.params,
      readTimeout: requestOptions.readTimeout || 5000,
    })
    if (response.status === 401) {
      clearStore()
      location.hash = '#/login'
    }
    // 在请求成功后执行的操作，比如处理响应数据
    console.log('Request successful:', response)
    span?.end()
    return response?.data
  } catch (error) {
    // 在请求失败后执行的操作，比如错误处理
    console.error('Request failed:', error)
    span?.end()
    throw error
  }
}

export default http
