import Logger from './logger'
import TCPMQTT from './tcp'
import { randomFrom } from './utils'
import Emitter from 'component-emitter'
import type { MQTTMessage } from '@capacitor-eshine/mqtt'

export default class MQTTClient {
  private static mqtt: TCPMQTT
  private static connectTimeout: NodeJS.Timeout
  private static publishTimeout: NodeJS.Timeout
  private static isConnected = false
  private static connectRetries = 0
  private static publishFails: {
    topic: string
    payload: any
    created: number
  }[] = []
  private static topics: Record<string, boolean> = {}
  private static connectionListener: (error?: string) => void
  private static logger = new Logger('MQTTClient')
  private static emitter = new Emitter()

  /**
   * MQTTClient
   * @param url mqtt服务url
   * @param port mqtt服务端口
   * @param clientId 客户端id
   * @param success 成功
   * @param error 失败
   * @param connectionLost 断开连接
   */
  static init({
    url,
    port,
    clientId,
    token,
  }: {
    url: string
    port?: number
    clientId: string
    token: string
  }) {
    if (!url || !clientId) throw new Error('url, clientId is required')
    this.logger.i('init', { url, port, clientId })
    // const _MQTT = window.cordova ? MQTT : WebMQTT;
    this.mqtt = new TCPMQTT(url, clientId, token, port)
    const online = this.online.bind(this)
    window.removeEventListener('online', online)
    window.addEventListener('online', online)
  }
  /**
   * 连接mqtt
   * @param connectionListener 连接状态监听
   */
  static connect(connectionListener?: (error?: string) => void) {
    connectionListener && (this.connectionListener = connectionListener)
    this.logger.i('connect start')
    if (!this.mqtt) {
      this.logger.w('MQTTClient uninitialized')
      return Promise.reject(new Error('MQTTClient uninitialized'))
    }
    return this.mqtt.connect((data) => {
      let { type, topic, payload } = (data || {}) as MQTTMessage
      switch (type) {
        case 'messageArrived':
          try {
            if (payload && typeof payload === 'string') {
              payload = JSON.parse(payload)
            }
          } catch (error) {}
          this.emitter.emit(topic, payload)
          break
        case 'connected':
          this.logger.i('connect successful')
          this.isConnected = true
          this.connectRetries = 0
          this.connectionListener && this.connectionListener()
          // 重新订阅
          Object.keys(this.topics).forEach((topic) => {
            this.mqtt.subscribe(topic)
          })
          // 重发未发送的消息
          this.republish()
          break
        case 'connectionLost':
          this.logger.e(data.type, data)
          this.isConnected = false
          this.connectionListener && this.connectionListener(data.type)
          this.reconnect()
          break
        case 'disconnect':
          this.logger.e(data.type, data)
          this.isConnected = false
          this.connectionListener && this.connectionListener(data.type)
          break
        default:
          this.logger.w('unkonw message', data)
          break
      }
    })
  }
  /**
   * 订阅消息
   * @param topic 主题
   * @param listen 监听
   */
  static subscribe(topic: string, listen: (payload: any) => void) {
    if (!topic) throw new Error('topic is required')
    this.logger.i('subscribe start', { topic })
    if (!this.mqtt) {
      this.logger.w('MQTTClient uninitialized')
      return Promise.reject(new Error('MQTTClient uninitialized'))
    }
    this.emitter.on(topic, listen)
    this.topics[topic] = true
    return this.mqtt
      .subscribe(topic)
      .then(() => {
        this.logger.i('subscribe successful', { topic })
      })
      .catch((e) => {
        this.logger.e('subscribe failed', { topic, error: e })
        return Promise.resolve(e)
      })
  }
  /**
   * 发送消息
   * @param topic 主题
   * @param payload 消息内容
   * @param success 成功
   * @param error 失败
   */
  static publish(topic: string, payload: any) {
    if (!topic || !payload) throw new Error('topic, payload is required')
    this.logger.i('publish start', { topic, payload })
    if (!this.mqtt) {
      this.logger.w('MQTTClient uninitialized')
      return Promise.reject(new Error('MQTTClient uninitialized'))
    }
    if (!this.isConnected) {
      this.publishFails.push({ topic, payload, created: Date.now() })
      return Promise.resolve()
    }
    try {
      if (typeof payload === 'object') {
        payload = JSON.stringify(payload)
      }
    } catch (error) {}
    return this.mqtt
      .publish(topic, payload)
      .then(() => {
        this.logger.i('publish successful', { topic })
        this.republish()
      })
      .catch((e) => {
        this.logger.e('publish failed', { topic, payload, error: e })
        this.publishFails.push({ topic, payload, created: Date.now() })
        this.disconnect()
        setTimeout(() => {
          this.connect()
        }, 500)
        // this.republish();
      })
  }
  /**
   * 取消订阅
   * @param topic 主题
   */
  static unsubscribe(topic: string) {
    if (!topic) throw new Error('topic is required')
    if (!this.mqtt) {
      this.logger.w('MQTTClient uninitialized')
      return Promise.reject(new Error('MQTTClient uninitialized'))
    }
    delete this.topics[topic]
    return this.mqtt
      .unsubscribe(topic)
      .then(() => {
        this.logger.i('unsubscribe successful', { topic })
      })
      .catch((e) => {
        this.logger.e('unsubscribe failed', { topic, error: e })
      })
  }
  /**
   * 断开连接
   */
  static disconnect() {
    this.logger.i('disconnect start')
    if (!this.mqtt) {
      this.logger.w('MQTTClient uninitialized')
      return Promise.reject(new Error('MQTTClient uninitialized'))
    }
    clearTimeout(this.publishTimeout)
    clearTimeout(this.connectTimeout)
    this.isConnected = false
    return this.mqtt
      .disconnect()
      .then((s) => {
        this.logger.i('disconnect successful', JSON.stringify(s))
      })
      .catch((e) => {
        this.logger.e('disconnect failed', JSON.stringify(e))
      })
  }
  /**
   * 重连
   */
  static reconnect() {
    clearTimeout(this.connectTimeout)
    this.connectTimeout = setTimeout(
      () => {
        if (this.isConnected) {
          return this.logger.w('is connected, in timeout function ')
        }
        this.connectRetries++
        this.logger.i('reconnect', {
          connectRetries: this.connectRetries,
          onLine: navigator.onLine,
        })
        this.connect()
      },
      randomFrom(2000, 5000) + this.connectRetries * 1000,
    )
  }
  /**
   * 重试publish
   */
  static republish() {
    clearTimeout(this.publishTimeout)
    if (!this.publishFails || !this.publishFails.length) return
    if (this.isConnected) {
      this.publishTimeout = setTimeout(() => {
        const _a = this.publishFails
        const msg = _a?.shift?.()
        if (!msg) return
        const { topic, payload } = msg
        this.logger.i('republish', {
          topic,
          payload,
          onLine: navigator.onLine,
          failCount: this.publishFails.length,
        })
        this.publish(topic, payload)
      }, 200)
    } else {
      this.logger.i('cancel republish', {
        isConnected: this.isConnected,
        onLine: navigator.onLine,
        failCount: this.publishFails.length,
      })
    }
  }
  /**
   * 恢复网络
   */
  static online() {
    clearTimeout(this.connectTimeout)
    if (!this.isConnected) {
      this.connectRetries = 0
      this.reconnect()
    }
  }
  /**
   * MQTT连接状态监听
   */
  static setConnectionListener(listener: (error?: string) => void) {
    this.connectionListener = listener
  }
  // 取消监听
  static removeListener(topic: string, listen: (payload: any) => void) {
    this.emitter.off(topic, listen)
  }
}
