/// <reference types="vitest" />

import legacy from '@vitejs/plugin-legacy'
import react from '@vitejs/plugin-react'
import requireTransform from 'vite-plugin-require-transform'
import { defineConfig } from 'vite'
import path from 'path'
import autoprefixer from 'autoprefixer'
import topLevelAwait from 'vite-plugin-top-level-await'
import { reactScopedCssPlugin } from 'rollup-plugin-react-scoped-css'
import * as BuildInfo from './scripts/version'
import { federation } from '@module-federation/vite'
import { ModuleFederationOptions } from '@module-federation/vite/lib/utils/normalizeModuleFederationOptions'
import fs from 'fs'
// 当前分支
const branchName = process.env.CI_COMMIT_REF_SLUG // execSync('git rev-parse --abbrev-ref HEAD').toString().trim()
console.log('branchName', branchName)

const proxyOption = {
  target: 'https://mobile-hq.downtown8.net/',
  changeOrigin: true,
  secure: false,
}

// source map url
const vitePluginSourceMap = () => {
  return {
    name: 'vite-plugin-sourcemap',
    apply: 'build',
    enforce: 'post',
    generateBundle(options, bundle) {
      for (const fileName in bundle) {
        const chunk = bundle[fileName]
        if (chunk.type === 'chunk' && typeof chunk.code === 'string') {
          if (!chunk.code.includes('sourceMappingURL')) {
            chunk.code += `//# sourceMappingURL=${fileName}.map\n`
          }
        }
      }
    },
  }
}

// 动态生成带时间戳的 URL
function generateTimestampedURL(baseUrl: string): string {
  const timestamp = new Date().getTime()
  return `${baseUrl}?ts=${timestamp}`
}

const remoteNames = [
  'mainline',
  'menu',
  'user',
  'setting',
  'login',
  'supply',
  'auth', // 授权模块
  'report',
]

const remotes = remoteNames.reduce(
  (result, remoteName) => {
    result[remoteName] = {
      type: 'module',
      name: remoteName,
      entry: generateTimestampedURL(`/${remoteName}/static/remoteEntry.js`),
      // entry:
      //   remoteName === 'report'
      //     ? generateTimestampedURL(
      //         'http://192.168.7.101:4173/report/static/remoteEntry.js',
      //       )
      //     : generateTimestampedURL(`/${remoteName}/static/remoteEntry.js`),
      entryGlobalName: remoteName,
      shareScope: 'default',
    }
    return result
  },
  {} as ModuleFederationOptions['remotes'],
)

const remoteProxy = remoteNames.reduce((result, name) => {
  // if (name === 'report') {
  //   result[`/${name}`] = {
  //     target: 'http://192.168.7.101:4173',
  //     changeOrigin: true,
  //     secure: false,
  //   }
  // } else {
  //   result[`/${name}`] = proxyOption
  // }
  result[`/${name}`] = proxyOption
  return result
}, {})

export default defineConfig({
  plugins: [
    // process.env.NODE_ENV === 'development' && esbuild({
    //   target: 'chrome64',
    //   include: /\.[jt]sx?$/,
    //   exclude: /node_modules/,
    //   loaders: {
    //     '.tsx': 'tsx',
    //     '.ts': 'ts'
    //   }
    // }),
    react(),
    legacy({
      // 去掉IE,移动端不会出现IE情况
      targets: ['defaults', 'not IE 11'],
    }),
    // viteCompression({
    //   verbose: true,
    //   disable: false,
    //   threshold: 10240,
    //   algorithm: 'gzip',
    //   ext: '.gz',
    // }),
    federation({
      dev: true,
      manifest: true,
      name: 'main',
      shareStrategy: 'loaded-first',
      /** 注册外部组件入口文件 */
      remotes,
      exposes: {},
      shared: {
        react: {
          singleton: true,
        },
        'react-dom': {
          singleton: true,
        },
        'react-router': {
          singleton: true,
        },
        'react-router-dom': {
          singleton: true,
        },
        '@ionic/react': {
          singleton: true,
        },
        mobx: {
          singleton: true,
        },
        'mobx-react': {
          singleton: true,
        },
        '@apollo/client': {
          singleton: true,
        },
      },
      runtimePlugins: [
        path.resolve(__dirname, './src/runtime-plugin/upateRemote.ts'),
      ],
    }),
    requireTransform({
      fileRegex: /.ts$|.tsx$/,
    }),
    topLevelAwait({
      // The export name of top-level await promise for each chunk module
      promiseExportName: '__tla',
      // The function to generate import names of top-level await promise in each chunk module
      promiseImportName: (i) => `__tla_${i}`,
    }),
    // @ts-ignore
    reactScopedCssPlugin({
      exclude: /node_modules/,
      hashPrefix: 'dt-main',
      scopeStyleByDefault: true,
    }),
    // vitePluginSourceMap(),
    // sentry必须在最后
    // sentryVitePlugin({
    //   org: 'downtown',
    //   project: 'mobile-hq',
    //   // errorHandler: (err)=> {
    //   //   console.log('err', err)
    //   //   process.exit(0)
    //   // },
    //   disable: branchName !== 'production',
    //   url: 'https://sentry.eshine.cn',
    //   // Auth tokens can be obtained from https://sentry.io/orgredirect/organizations/:orgslug/settings/auth-tokens/
    //   authToken:
    //     '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',
    // }),
  ],
  server: {
    port: 5173, // 将端口设置为5200
    open: true,
    host: '0.0.0.0',
    proxy: {
      '/api/graphql': proxyOption,
      '/new-api/graphql': proxyOption,
      ...remoteProxy,
      // '/mainline': proxyOption,
      // '/menu': proxyOption,
      // '/setting': proxyOption,
      // '/login': proxyOption,
      // '/user': proxyOption,
      // '/supply': proxyOption,
    },
  },
  preview: {
    port: 4000, // 将端口设置为5200
    open: true,
    host: '0.0.0.0',
    proxy: {
      '/api/graphql': proxyOption,
      '/mainline': proxyOption,
      // '/mainline': {
      //   target: 'http://192.168.7.66:3100',
      //   changeOrigin: true,
      //   secure: false,
      // },
      // '/menu': {
      //   target: 'http://192.168.7.84:4173',
      //   changeOrigin: true,
      //   secure: false,
      // },
      '/menu': proxyOption,
      '/setting': proxyOption,
      '/login': proxyOption,
      '/user': proxyOption,
      '/supply': proxyOption,
      '/new-api/graphql': proxyOption,
      '/auth': proxyOption, // 授权模块
    },
  },
  test: {
    globals: true,
    environment: 'jsdom',
    setupFiles: './src/setupTests.ts',
  },
  resolve: {
    alias: {
      '@api': path.resolve(__dirname, 'src/api'),
      '@components': path.resolve(__dirname, 'src/components'),
      '@mobx': path.resolve(__dirname, 'src/mobx'),
      '@gqlDocuments': path.resolve(__dirname, 'src/gqlDocuments'),
      '@gql': path.resolve(__dirname, 'src/gql'),
      '@pages': path.resolve(__dirname, 'src/pages'),
      '@config': path.resolve(__dirname, 'src/config'),
      '@common': path.resolve(__dirname, 'src/common'),
      '@router': path.resolve(__dirname, 'src/router'),
      '@utils': path.resolve(__dirname, 'src/common/utils'),
      '@env': path.resolve(__dirname, 'src/common/env'),
      '@couchbase': path.resolve(__dirname, 'src/couchbase'),
      '@init': path.resolve(__dirname, 'src/init'),
      '@cps': path.resolve(__dirname, 'src/components'),
      '@sdk': path.resolve(__dirname, 'src/sdk'),
      '@css': path.resolve(__dirname, 'src/assets/css'),
      '@img': path.resolve(__dirname, 'src/assets/img'),
      '@': path.resolve(__dirname, 'src'),
    },
  },
  css: {
    preprocessorOptions: {
      scss: {
        additionalData: `@import "${path.resolve(__dirname, 'node_modules/@dt/ui-mobile/dist/style/input.scss')}";`,
      },
    },
    postcss: {
      plugins: [
        autoprefixer(),
        // postcsspxtoviewport({
        //   viewportWidth: 412, // 视窗的宽度，对应的是我们设计稿的宽度，一般是375
        //   unitPrecision: 3, // 指定`px`转换为视窗单位值的小数位数（很多时候无法整除）
        //   viewportUnit: 'vw', // 指定需要转换成的视窗单位，建议使用vw
        //   selectorBlackList: [], // 指定不转换为视窗单位的类，可以自定义，可以无限添加,建议定义一至两个通用的类名
        //   minPixelValue: 1, // 小于或等于`1px`不转换为视窗单位，你也可以设置为你想要的值
        //   mediaQuery: false, // 允许在媒体查询中转换`px`
        // })
      ],
    },
  },
  define: {
    buildInfo: BuildInfo,
  },
  build: {
    chunkSizeWarningLimit: 2048, // Set limit to 10000KB
    assetsDir: 'static',
    sourcemap: true,
    rollupOptions: {
      output: {
        sourcemapExcludeSources: true, // 排除特定库的 source map
        sourcemapIgnoreList: (path) => path.includes('node_modules'),
      },
      plugins: [
        {
          name: 'generate-custom-file',
          writeBundle() {
            const filePath = path.resolve('dist', 'version.json')
            const content = JSON.stringify(BuildInfo)
            fs.writeFileSync(filePath, content, 'utf-8')
            console.log('✅ Custom file generated:', filePath, content)
          },
        },
      ],
    },
  },
  optimizeDeps: {
    include: ['swiper/react', 'swiper/modules'],
  },
  // experimental: {
  //   renderBuiltUrl(filename, { hostId, hostType, type }) {
  //     if (hostType !== 'html') {
  //       return {
  //         runtime:
  //           `'https://static.' + (location.host.split('.').splice(1, location.host.split('.').length -1).join('.') || 'downtown8.net') + '/mobile-hq/'+` +
  //           JSON.stringify(filename),
  //       }
  //     } else {
  //       return { relative: true }
  //     }
  //   },
  // },
})
